<?php

namespace LakeDrops\DockerTraefik;

use Symfony\Component\Filesystem\Filesystem;
use Symfony\Component\Yaml\Yaml;

class Traefik {

  /** @var string */
  protected $name;

  /** @var Filesystem */
  protected $fs;

  /**
   * Traefik constructor.
   *
   * @param string $name
   *  Name of the network to be created which needs to match the container
   *  prefix of your project you would like to handle with Traefik.
   */
  public function __construct($name) {
    $this->name = $name;
    $this->fs = new Filesystem();
  }

  /**
   * Update the Traefik container.
   */
  public function update() {
    // Update host wider traefik container
    $traefikPath = $_SERVER['HOME'] . '/.traefik';
    $traefikFile = $traefikPath . '/docker-compose.yml';

    if ($this->fs->exists($traefikFile)) {
      $traefik = Yaml::parse(file_get_contents($traefikFile));
    }
    else {
      $this->fs->mkdir($traefikPath);
      $traefik = [
        'version' => '2',
        'services' => [
          'traefik' => [
            'image' => 'traefik',
            'restart' => 'unless-stopped',
            'command' => '-c /dev/null --web --docker --logLevel=DEBUG',
            'networks' => [],
            'ports' => [
              '8000:80',
              '8080:8080',
            ],
            'volumes' => [
              '/var/run/docker.sock:/var/run/docker.sock',
            ],
          ],
          'portainer' => [
            'image' => 'portainer/portainer',
            'restart' => 'unless-stopped',
            'command' => '--no-auth',
            'volumes' => [
              '/var/run/docker.sock:/var/run/docker.sock',
            ],
            'ports' => [
              '9000:9000',
            ],
          ],
        ],
        'networks' => [],
      ];
    }

    if (!in_array($this->name, $traefik['services']['traefik']['networks'])) {
      $traefik['services']['traefik']['networks'][] = $this->name;
      $traefik['networks'][$this->name] = [
        'external' => [
          'name' => $this->name . '_default',
        ],
      ];
      file_put_contents($traefikFile, Yaml::dump($traefik, 9, 2));
      exec('docker network create ' . $this->name . '_default');
      exec('docker-compose -f ' . $traefikFile . ' --project-name traefik stop');
      exec('docker-compose -f ' . $traefikFile . ' --project-name traefik up -d');
    }
  }

}
